/*
 *  Copyright 2019, Antoine Fontaine <antoine.fontaine@epfl.ch>
 *  I release this program under GNU GPLv3+.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "gui-downloadable-file.h"
#include "data-struct.h"
#include "moodle.h"
#include "config.h"

#define HANDY_USE_UNSTABLE_API
#include <handy.h>

struct _GuiDownloadableFile
{
  HdyActionRow parent_instance;
  // contains no widgets
  guint id;
  gboolean downloaded;
};

enum {
  PROP_0, PROP_FILE_NAME, PROP_FILE_TEXTNAME, PROP_FILE_ID, N_FILE_PROPERTIES
};

GParamSpec *gui_downloadable_file_properties[N_FILE_PROPERTIES] = { NULL, };

G_DEFINE_TYPE (GuiDownloadableFile, gui_downloadable_file, HDY_TYPE_ACTION_ROW)


GuiDownloadableFile *
gui_downloadable_file_new (struct DownloadableFile file_info)
{
  return g_object_new (HDY_TYPE_GUI_DOWNLOADABLE_FILE,
                       "file_name", file_info.name,
                       "file_textname", file_info.textname,
                       "file_id", file_info.id,
                       NULL);
}

static void
open_file_cb (gpointer ptr, const char *filename)
{
  //GuiDownloadableFile *self = ptr;
  char filepath[200];
  snprintf (filepath, sizeof (filepath), "file:///"CACHE_ROOT"/%s", filename);
  if (!g_app_info_launch_default_for_uri (filepath, NULL, NULL)) {
    g_warning ("Couldn't start a program to open '%s'\n", filepath);
  }
}

void
activated_cb (GtkListBox* unused, GuiDownloadableFile *self)
{
  /*if (self->downloaded)
    open_file_cb (self);
  else*/
    moodle_provider_download_file (self->id, open_file_cb, self);
}

static void
gui_downloadable_file_set_property (GObject      *object,
                                    guint         property_id,
                                    const GValue *value,
                                    GParamSpec   *pspec)
{
  GuiDownloadableFile *self = HDY_GUI_DOWNLOADABLE_FILE (object);

  switch (property_id) {
  case PROP_FILE_NAME:
    hdy_action_row_set_title (&self->parent_instance, g_value_get_string (value));
    break;
  case PROP_FILE_TEXTNAME:
    hdy_action_row_set_subtitle (&self->parent_instance, g_value_get_string (value));
    break;
  case PROP_FILE_ID:
    self->id = g_value_get_uint (value);
    break;
  default:
    // We don't have any other property...
    G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
    break;
  }
}

static void
gui_downloadable_file_class_init (GuiDownloadableFileClass *class)
{
  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
  GObjectClass *object_class = G_OBJECT_CLASS (class);

  gui_downloadable_file_properties[PROP_FILE_NAME] =
    g_param_spec_string ("file_textname", "textname",
                         "The file textname, like 'Exercices Week 6'",
                         NULL,
                         G_PARAM_WRITABLE|G_PARAM_CONSTRUCT_ONLY);

  gui_downloadable_file_properties[PROP_FILE_TEXTNAME] =
    g_param_spec_string ("file_name", "name",
                         "The file name, like 'exercises-w6.pdf'",
                         NULL,
                         G_PARAM_WRITABLE|G_PARAM_CONSTRUCT_ONLY);

  gui_downloadable_file_properties[PROP_FILE_ID] =
    g_param_spec_uint ("file_id", "id",
                         "The file id, like '102093'",
                         0, G_MAXUINT,
                         0,
                         G_PARAM_WRITABLE|G_PARAM_CONSTRUCT_ONLY);

  object_class->set_property = gui_downloadable_file_set_property;
  g_object_class_install_properties (object_class,
                                     N_FILE_PROPERTIES,
                                     gui_downloadable_file_properties);

  gtk_widget_class_set_template_from_resource (widget_class, "/sm/puri/handy/moodle/ui/gui-downloadable-file.ui");
}


static void
gui_downloadable_file_init (GuiDownloadableFile *self)
{
  gtk_widget_init_template (GTK_WIDGET (self));
}
