/*
 *  Copyright 2019, Antoine Fontaine <antoine.fontaine@epfl.ch>
 *  I release this program under GNU GPLv3+.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "gui-week.h"
#include "gui-downloadable-file.h"

#define HANDY_USE_UNSTABLE_API
#include <handy.h>

struct _GuiWeek
{
  GtkBox parent_instance;
  HdyActionRow *text;
  GtkLabel *week_label;
  guint week_number;
  GtkListBox *week_content;
  guint file_count;
  struct DownloadableFile* files;
};

enum {
  PROP_0, PROP_WEEK_NUMBER, PROP_WEEK_FILE_COUNT, PROP_WEEK_FILES, N_WEEK_PROPERTIES
};

GParamSpec *gui_week_properties[N_WEEK_PROPERTIES] = { NULL, };


G_DEFINE_TYPE (GuiWeek, gui_week, GTK_TYPE_BOX)


GuiWeek* gui_week_new (struct Week week_info)
{
  return g_object_new (HDY_TYPE_GUI_WEEK,
                       "week_number", week_info.week_number,
                       "file_count", week_info.file_count,
                       "files", week_info.files,
                       NULL);
}


static void
update_files (GuiWeek* self)
{
  for (guint i=0; i<self->file_count; i++) {
    GuiDownloadableFile* file = gui_downloadable_file_new (self->files[i]);
    gtk_list_box_insert (self->week_content, GTK_WIDGET (file), i+1);
  }
}

static void
gui_week_set_property (GObject      *object,
                       guint         property_id,
                       const GValue *value,
                       GParamSpec   *pspec)
{
  GuiWeek *self = HDY_GUI_WEEK (object);

  switch (property_id) {
  case PROP_WEEK_NUMBER:
    self->week_number = g_value_get_uint (value);
    { gchar* label = (self->week_number == 0
                        ? g_strdup ("General")
                        : g_strdup_printf ("Week %u", self->week_number));
      gtk_label_set_label (self->week_label, label); }
    break;
  case PROP_WEEK_FILE_COUNT:
    self->file_count = g_value_get_uint (value);
    break;
  case PROP_WEEK_FILES:
    self->files = g_value_get_pointer (value);
    //self->files+=2;
    update_files (self);
    break;
  //hdy_action_row_set_title (text, "Hi!");
  default:
    // We don't have any other property...
    G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
    break;
  }
}

static void
gui_week_class_init (GuiWeekClass *class)
{
  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
  GObjectClass *object_class = G_OBJECT_CLASS (class);

  gui_week_properties[PROP_WEEK_NUMBER] =
    g_param_spec_uint ("week_number", "week_number",
                       "The number of the week",
                       0, G_MAXUINT,
                       0,
                       G_PARAM_WRITABLE|G_PARAM_CONSTRUCT_ONLY);

  gui_week_properties[PROP_WEEK_FILE_COUNT] =
    g_param_spec_uint ("file_count", "file_count",
                       "The number of files the *files points to",
                       0, G_MAXUINT,
                       0,
                       G_PARAM_WRITABLE|G_PARAM_CONSTRUCT_ONLY);

  gui_week_properties[PROP_WEEK_FILES] =
    g_param_spec_pointer ("files", "files",
                          "The downloadable files",
                          G_PARAM_WRITABLE|G_PARAM_CONSTRUCT_ONLY);

  object_class->set_property = gui_week_set_property;
  g_object_class_install_properties (object_class,
                                     N_WEEK_PROPERTIES,
                                     gui_week_properties);

  gtk_widget_class_set_template_from_resource (widget_class, "/sm/puri/handy/moodle/ui/gui-week.ui");
  gtk_widget_class_bind_template_child (widget_class, GuiWeek, week_label);
  gtk_widget_class_bind_template_child (widget_class, GuiWeek, week_content);
  gtk_widget_class_bind_template_callback_full (widget_class, "file_selected_cb", G_CALLBACK(activated_cb));
}

static void
gui_week_init (GuiWeek *self)
{
  gtk_widget_init_template (GTK_WIDGET (self));
  gtk_list_box_set_header_func (self->week_content, hdy_list_box_separator_header, NULL, NULL);
}
