/*
 *  Copyright 2019, Antoine Fontaine <antoine.fontaine@epfl.ch>
 *  I release this program under GNU GPLv3+.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "gui-window.h"
#include "gui-course.h"
#include "data-struct.h"
#include "moodle.h"

#include <glib/gi18n.h>
#define HANDY_USE_UNSTABLE_API
#include <handy.h>

struct _GuiWindow
{
  GtkApplicationWindow parent_instance;

  HdyLeaflet *header_box;
  HdyLeaflet *content_box;
  GtkButton *back;
  //GtkButton *backTwice;
  //GtkToggleButton *search_button;
  GtkStackSidebar *sidebar;
  GtkStack *stack;
  struct Course* courses;
  guint course_count;
  //HdySearchBar *search_bar;
  //GtkEntry *search_entry;
  HdyHeaderGroup *header_group;
};

G_DEFINE_TYPE (GuiWindow, gui_window, GTK_TYPE_APPLICATION_WINDOW)

static gboolean
gui_window_key_pressed_cb (GtkWidget   *sender,
                           GdkEventKey *event,
                           GuiWindow   *self)
{
  GdkModifierType default_modifiers = gtk_accelerator_get_default_mod_mask ();
  guint keyval;
  GdkModifierType state;

  gdk_event_get_keyval ((GdkEvent *) event, &keyval);
  gdk_event_get_state ((GdkEvent *) event, &state);

  if ((keyval == GDK_KEY_q || keyval == GDK_KEY_Q) &&
      (state & default_modifiers) == GDK_CONTROL_MASK) {
    gtk_widget_destroy (GTK_WIDGET (self));

    return TRUE;
  }

  return FALSE;
}

static void
update (GuiWindow *self)
{
  GtkWidget *header_child = hdy_leaflet_get_visible_child (self->header_box);
  HdyFold fold = hdy_leaflet_get_fold (self->header_box);

  g_assert (header_child == NULL || GTK_IS_HEADER_BAR (header_child));

  hdy_header_group_set_focus (self->header_group, fold == HDY_FOLD_FOLDED ? GTK_HEADER_BAR (header_child) : NULL);
}

static void
update_header_bar (GuiWindow *self)
{
  //const gchar *visible_child_name;

  //visible_child_name = gtk_stack_get_visible_child_name (GTK_STACK (self->stack));
  //gtk_widget_set_visible (GTK_WIDGET (self->search_button),
  //                        g_str_equal (visible_child_name, "search-bar"));
}

static void
update_courses (GuiWindow *self)
{
  for (int i=0; i<self->course_count; i++) {
    GuiCourse *course = gui_course_new (self->courses[i]);
    gtk_stack_add_titled (self->stack,
                          GTK_WIDGET (course),
                          self->courses[i].name,
                          self->courses[i].name);
  }
}

static void
get_courses_cb (gpointer ptr, struct Courses courses)
{
  GuiWindow* self = ptr;
  self->courses = courses.courses;
  self->course_count = courses.course_count;
  update_courses (self);
}

static void
gui_window_notify_header_visible_child_cb (GObject    *sender,
                                           GParamSpec *pspec,
                                           GuiWindow  *self)
{
  update (self);
}

static void
gui_window_notify_fold_cb (GObject    *sender,
                           GParamSpec *pspec,
                           GuiWindow  *self)
{
  update (self);
}

static void
gui_window_notify_visible_child_cb (GObject    *sender,
                                    GParamSpec *pspec,
                                    GuiWindow  *self)
{
  hdy_leaflet_set_visible_child_name (self->content_box, "content");
  update_header_bar (self);
}

static void
gui_window_back_clicked_cb (GtkWidget *sender,
                            GuiWindow *self)
{
  hdy_leaflet_set_visible_child_name (self->content_box, "sidebar");
}


static void
dialog_close_cb (GtkDialog *self)
{
  gtk_widget_destroy (GTK_WIDGET (self));
}

static void
dialog_clicked_cb (GtkButton *btn,
                   GuiWindow *self)
{
  GtkWidget *dlg;
  GtkWidget *lbl;

  dlg = hdy_dialog_new (GTK_WINDOW (self));
  gtk_window_set_title (GTK_WINDOW (dlg), "HdyDialog");
  lbl = gtk_label_new ("Hello, World!");
  g_object_set (lbl, "margin", 12, NULL);
  gtk_widget_set_vexpand (lbl, TRUE);
  gtk_widget_set_valign (lbl, GTK_ALIGN_CENTER);
  gtk_widget_set_halign (lbl, GTK_ALIGN_CENTER);
  gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dlg))),
                     lbl);

  gtk_widget_show (lbl);
  gtk_widget_show (dlg);
}

static void
dialog_action_clicked_cb (GtkButton *btn,
                          GuiWindow *self)
{
  GtkWidget *dlg;
  GtkWidget *lbl;

  dlg = hdy_dialog_new (GTK_WINDOW (self));
  gtk_window_set_title (GTK_WINDOW (dlg), "HdyDialog");
  gtk_dialog_add_buttons (GTK_DIALOG (dlg),
                          "Done", GTK_RESPONSE_ACCEPT,
                          "Cancel", GTK_RESPONSE_CANCEL,
                          NULL);
  gtk_dialog_set_default_response (GTK_DIALOG (dlg), GTK_RESPONSE_ACCEPT);
  g_signal_connect (G_OBJECT (dlg), "response", G_CALLBACK (dialog_close_cb), NULL);
  lbl = gtk_label_new ("Hello, World!");
  g_object_set (lbl, "margin", 12, NULL);
  gtk_widget_set_vexpand (lbl, TRUE);
  gtk_widget_set_valign (lbl, GTK_ALIGN_CENTER);
  gtk_widget_set_halign (lbl, GTK_ALIGN_CENTER);
  gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dlg))),
                     lbl);

  gtk_widget_show (lbl);
  gtk_widget_show (dlg);
}

GuiWindow *
gui_window_new (GtkApplication *application)
{
  return g_object_new (HDY_TYPE_GUI_WINDOW, "application", application, NULL);
}


static void
gui_window_class_init (GuiWindowClass *class)
{
  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);

  gtk_widget_class_set_template_from_resource (widget_class, "/sm/puri/handy/moodle/ui/gui-window.ui");
  gtk_widget_class_bind_template_child (widget_class, GuiWindow, header_box);
  gtk_widget_class_bind_template_child (widget_class, GuiWindow, content_box);
  gtk_widget_class_bind_template_child (widget_class, GuiWindow, back);
  //gtk_widget_class_bind_template_child (widget_class, GuiWindow, search_button);
  gtk_widget_class_bind_template_child (widget_class, GuiWindow, sidebar);
  gtk_widget_class_bind_template_child (widget_class, GuiWindow, stack);
  //gtk_widget_class_bind_template_child (widget_class, GuiWindow, search_bar);
  //gtk_widget_class_bind_template_child (widget_class, GuiWindow, search_entry);
  gtk_widget_class_bind_template_child (widget_class, GuiWindow, header_group);
  gtk_widget_class_bind_template_callback_full (widget_class, "key_pressed_cb", G_CALLBACK(gui_window_key_pressed_cb));
  gtk_widget_class_bind_template_callback_full (widget_class, "notify_header_visible_child_cb", G_CALLBACK(gui_window_notify_header_visible_child_cb));
  gtk_widget_class_bind_template_callback_full (widget_class, "notify_fold_cb", G_CALLBACK(gui_window_notify_fold_cb));
  gtk_widget_class_bind_template_callback_full (widget_class, "notify_visible_child_cb", G_CALLBACK(gui_window_notify_visible_child_cb));
  gtk_widget_class_bind_template_callback_full (widget_class, "back_clicked_cb", G_CALLBACK(gui_window_back_clicked_cb));
  gtk_widget_class_bind_template_callback_full (widget_class, "dialog_clicked_cb", G_CALLBACK(dialog_clicked_cb));
  gtk_widget_class_bind_template_callback_full (widget_class, "dialog_action_clicked_cb", G_CALLBACK(dialog_action_clicked_cb));
}


static void
gui_window_init (GuiWindow *self)
{
  self->course_count = 0;
  gtk_widget_init_template (GTK_WIDGET (self));

  moodle_provider_init(&get_courses_cb, MOODLE_MODE_ONLINE, self);

  hdy_leaflet_set_visible_child_name (self->content_box, "content");
  update_header_bar (self);
}
