/*
 *  Copyright 2019, Antoine Fontaine <antoine.fontaine@epfl.ch>
 *  I release this program under GNU GPLv3+.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <libsoup/soup.h>
#include "moodle.h"
#include "moodle-parser.h"
#include "password.h"
#include "utils.h"
#include "config.h"

#define MOODLE_FILE_INDEX CACHE_ROOT"/index.html"

struct course_content_cb_data {
  course_content_cb cb;
  gpointer          user_data;
  gchar            *filename;
};
struct file_cb_data {
  file_cb      cb;
  gpointer     user_data;
  SoupMessage *msg;
};

static struct {
  SoupSession *session;
  SoupCookieJar *jar;
  SoupLogger *logger;
  MoodleOnlineMode online_mode;
} self;


static void
moodle_course_content_cb (SoupSession *session,
                          SoupMessage *msg,
                          gpointer     course_cb_ptr)
{
  struct course_content_cb_data *course_cb = course_cb_ptr;
  // TODO: check if all is fine
  if (msg->status_code != 200)
      g_warning ("connection error, http %u\n", msg->status_code);

  SoupBuffer *result = soup_message_body_flatten (msg->response_body);

  struct CourseContent content = moodle_parse_course_content (result->data, result->length, course_cb->filename);

  course_cb->cb(course_cb->user_data, content);
  free (course_cb->filename);
  free (course_cb);
  moodle_parser_course_content_free (content);
}


static void
moodle_downloaded_file_cb (GObject      *unused,
                           GAsyncResult *res,
                           gpointer      ptr)
{
  struct file_cb_data *cb_info = ptr;
  GInputStream *is = soup_session_send_finish (self.session,
                                               res,
                                               NULL);

  const char *filename = g_path_get_basename (soup_uri_to_string (soup_message_get_uri (cb_info->msg), FALSE));
  write_g_input_stream_to_file (is, filename);
  cb_info->cb (cb_info->user_data, filename);
  free (cb_info);
}

void
moodle_provider_download_file (guint id, file_cb cb, gpointer cb_arg)
{
  char url[strlen (EPFL_FILE_LINK)+50];
  sprintf (url, EPFL_FILE_LINK"%u&redirect=1", id);
  SoupMessage *msg = soup_message_new ("GET", url);
  soup_message_set_priority (msg, SOUP_MESSAGE_PRIORITY_HIGH);
  struct file_cb_data *cb_info = malloc (sizeof (struct file_cb_data));
  cb_info->cb = cb;
  cb_info->user_data = cb_arg;
  cb_info->msg = msg;
  soup_session_send_async (self.session,
                           msg,
                           NULL,
                           moodle_downloaded_file_cb,
                           (gpointer) cb_info);
}


static enum moodle_error
moodle_provider_connect (GInputStream **response)
{
  SoupMessage *msg;
  char request_key[33];
  /*char *error;
  char *html;
  size_t html_size;*/

  msg = soup_message_new ("GET", EPFL_TEQUILA_LOGIN);
  soup_session_send_message (self.session, msg);
  if (msg->status_code != 200)
    return MOODLE_ERROR_CONNECTION;


  char *username=USERNAME, *password;
  if (!moodle_password_get (&password))
    g_error ("couldn't get password\n");
  // read request key, needed to auth
  gchar *uri = soup_uri_to_string (soup_message_get_uri (msg), FALSE);
  sscanf (strstr (uri, "requestkey="), "requestkey=%32s", request_key);
  msg = soup_form_request_new ("POST",
                               EPFL_TEQUILA_LOGIN_ACTION,
                               "requestkey", request_key,
                               "username", username,
                               "password", password,
                               NULL);
  *response = soup_session_send (self.session,
                                 msg,
                                 NULL,
                                 NULL);
  if (msg->status_code != 200)
    return MOODLE_ERROR_AUTH;
  //read_g_input_stream (*response, &html, &html_size);
  /*if ((error = moodle_parse_form_error (html, html_size))) {
    g_warning ("Authentification error: %s", error);
    free (html);
    return MOODLE_ERROR_AUTH;
  }
  free (html);*/

  if (strncmp (soup_uri_to_string (soup_message_get_uri (msg), FALSE),
               EPFL_TEQUILA_LOGIN_ACTION,
               strlen(EPFL_TEQUILA_LOGIN_ACTION)) == 0)
    g_error ("Did not redirect. This is due an incorrect password, or a bug.\n");

  g_print ("Authentificated :-)\n");

  //moodle_password_free (password);
  g_object_unref (msg);
  return MOODLE_ERROR_OK;
}


enum moodle_error
moodle_provider_get_course_content (course_content_cb cb, guint id, gpointer cb_arg)
{
  char filename[100];
  char url[100];
  sprintf(filename, CACHE_ROOT"/course-%u.html", id);
  sprintf(url, EPFL_COURSE_LINK"%u", id);

  if (self.online_mode == MOODLE_MODE_ONLINE) {

    SoupMessage *msg;
    msg = soup_message_new ("GET", url);
    soup_message_set_priority (msg, SOUP_MESSAGE_PRIORITY_LOW);
    struct course_content_cb_data *course_cb = malloc (sizeof (struct course_content_cb_data));
    course_cb->cb = cb;
    course_cb->user_data = cb_arg;
    course_cb->filename = malloc (strlen (filename)+1);
    strcpy (course_cb->filename, filename);
    soup_session_queue_message (self.session, msg, moodle_course_content_cb, (gpointer)course_cb);

  } else { // offline_mode

    GInputStream *is;
    GFile *file = g_file_new_for_path (filename);
    char  *html;
    size_t html_size;
    is = G_INPUT_STREAM (g_file_read (file, NULL, NULL));
    g_object_unref (file);
    if (!G_IS_INPUT_STREAM (is))
      g_error ("Could not read the course id %u (%s) from cache\n", id, filename);

    read_g_input_stream (is, &html, &html_size);
    struct CourseContent content = moodle_parse_course_content (html, html_size, filename);
    free (html);
    cb(cb_arg, content);
    moodle_parser_course_content_free (content);

  }

  return MOODLE_ERROR_OK;
}


enum moodle_error
moodle_provider_init (courses_cb cb, MoodleOnlineMode mode, gpointer cb_arg)
{
  self.online_mode = mode;
  GInputStream *index;
  char *html;
  size_t html_size;

  moodle_parser_init (CACHING && self.online_mode==MOODLE_MODE_ONLINE);

  if (self.online_mode == MOODLE_MODE_ONLINE) {

    self.session = soup_session_new_with_options (SOUP_SESSION_MAX_CONNS_PER_HOST, 5,
                                                  SOUP_SESSION_MAX_CONNS, 5,
                                                  SOUP_SESSION_SSL_STRICT, TRUE,
                                                  NULL);

    self.jar = soup_cookie_jar_new ();
    soup_session_add_feature (self.session,
                              SOUP_SESSION_FEATURE (self.jar));

    // libsoup logging
    if (LOGGING_REQUESTS) {
      if (LOGGING_REQUESTS == 1)
        self.logger = soup_logger_new (SOUP_LOGGER_LOG_HEADERS, 0);
      else if (LOGGING_REQUESTS == 2)
        self.logger = soup_logger_new (SOUP_LOGGER_LOG_BODY, 100);
      else
        self.logger = soup_logger_new (SOUP_LOGGER_LOG_BODY, 1000000);
      soup_session_add_feature (self.session,
                                SOUP_SESSION_FEATURE (self.logger));
    }

    switch (moodle_provider_connect (&index)) {
      case MOODLE_ERROR_OK:
        break;
      case MOODLE_ERROR_CONNECTION:
        g_error ("Could not reach Moodle");
      case MOODLE_ERROR_AUTH:
        g_error ("Could not authenticate");
      default:
        g_assert_false ("Should never happend");
    }

  } else { // offline mode

    GFile *file = g_file_new_for_path (MOODLE_FILE_INDEX);
    index = G_INPUT_STREAM (g_file_read (file, NULL, NULL));
    if (!G_IS_INPUT_STREAM (file))
    //  g_error ("No input file '%s'.\n", MOODLE_FILE_INDEX);
    g_object_unref (file);

  }

  read_g_input_stream (index, &html, &html_size);
  struct Courses c = moodle_parse_course_list (html, html_size, MOODLE_FILE_INDEX);
  free (html);
  cb(cb_arg, c);

  for (size_t i=0; i<c.course_count; i++) {
    free (c.courses[i].url);
    free (c.courses[i].name);
  }
  free (c.courses);

  return MOODLE_ERROR_OK;
}

