/*
 *  Copyright 2019, Antoine Fontaine <antoine.fontaine@epfl.ch>
 *  I release this program under GNU GPLv3+.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <sys/mman.h>  // mmap
#include <fcntl.h>     // open
#include <gio/gio.h>   // GInputStream stuff
#include "utils.h"

#define CHUNK_SIZE 100000


void
read_file (const char *name, char **content, size_t *size)
{
  // https://stackoverflow.com/questions/174531/how-to-read-the-content-of-a-file-to-a-string-in-c
  // simpler, but unportable to, say, Windows
  int fd = open(name, O_RDONLY);
  *size = lseek(fd, 0, SEEK_END);
  *content = mmap(0, *size, PROT_READ, MAP_PRIVATE, fd, 0);
}


void
write_file (const char *name, const char *content, size_t size)
{
  FILE *file = fopen (name, "wb");
  if (fwrite (content, sizeof(char), size, file) != size)
    g_warning ("weird number of written char");
  fclose (file);
}


void
read_g_input_stream (GInputStream *is, char **content, size_t *size)
{
  char buffer[CHUNK_SIZE];
  size_t buffer_size;

  size_t used_size=0;
  size_t allocated_size = CHUNK_SIZE;
  *content = malloc (CHUNK_SIZE);
  *size = 0;

  while ((buffer_size = g_input_stream_read (is, buffer, CHUNK_SIZE, NULL, NULL))) {
    if (buffer_size==-1)
      g_error ("Error reading input stream %s", buffer);

    if (used_size+buffer_size >= allocated_size) {
      if (allocated_size<1e6)
        allocated_size *= 2;
      *content = realloc (*content, allocated_size);
    }
    memcpy (used_size+*content, buffer, buffer_size);
    used_size += buffer_size;
  }

  g_object_unref (is);
  *content = realloc (*content, used_size+1);
  //*content[used_size+1] = '\0';
  *size = used_size;
}


void
write_g_input_stream_to_file (GInputStream *is, const char *filename)
{
  char buffer[CHUNK_SIZE];
  size_t size;
  FILE *file = fopen (filename, "wb");

  while ((size = g_input_stream_read (is, buffer, CHUNK_SIZE, NULL, NULL))) {
    if (size==-1)
      g_error ("Error reading input stream %s", buffer);
    if (fwrite (buffer, sizeof(char), size, file) != size)
      g_warning ("weird number of written char");
  }
  g_object_unref (is);
  fclose (file);
}
